@@ -4,7 +4,7 @@ use v5.14;
 use warnings;
 use utf8;
 
-our $VERSION = "2.24";
+our $VERSION = "2.25";
 
 use Data::Dumper;
 $Data::Dumper::Sortkeys = 1;
@@ -292,20 +292,24 @@ sub fold {
     my $eol = '';
     my $room = $width;
     @bg_stack = @color_stack = @reset = ();
-    my $yield_re = $opt->{expand} ? qr/[^\e\n\f\r\t]/ : qr/[^\e\n\f\r]/;
+    my $unremarkable_re =
+	$opt->{expand} ? qr/[^\e\n\f\r\0\N{U+2028}\N{U+2029}\t]/
+		       : qr/[^\e\n\f\r\0\N{U+2028}\N{U+2029}]/;
 
   FOLD:
     while (length) {
 
-	# newline
-	if (s/\A(\r*\n)//) {
+	# newline, null
+	# U+2028: Line Separator
+	# U+2029: Paragraph Separator
+	if (s/\A(\r*\n|[\0\N{U+2028}\N{U+2029}])//) {
 	    $eol = $1;
 	    last;
 	}
-	# formfeed
-	if (/\A(\f+)(.*)/s) {
+	# form feed
+	if (m/\A(\f+)/p) {
 	    last if length $folded;
-	    ($folded, $_) = ($1, $2);
+	    ($folded, $_) = ($1, ${^POSTMATCH});
 	    next;
 	}
 	# carriage return
@@ -355,25 +359,20 @@ sub fold {
 
 	# backspace
 	my $bs = 0;
-	while (s/\A(?:\X\cH+)++(?<c>\X|\z)//p) {
+	while (m/\A(?:\X\cH+)++(?<c>\X|\z)/p) {
 	    my $w = vwidth($+{c});
-	    if ($w > $room) {
-		if ($folded eq '') {
-		    $folded .= ${^MATCH};
-		    $room -= $w;
-		} else {
-		    $_ = ${^MATCH} . $_;
-		}
-		last FOLD;
-	    }
+	    last FOLD if $w > $room and $room != $width;
 	    $folded .= ${^MATCH};
+	    $_ = ${^POSTMATCH};
 	    $room -= $w;
 	    $bs++;
-	    last if $room < 1;
+	    last FOLD if $room < 0;
+	    last      if $room < 1;
 	}
 	next if $bs;
 
-	if (s/\A(\e+|(?:${yield_re}(?!\cH))+)//) {
+	# consume unremarkable characters
+	if (s/\A(\e+|(?:${unremarkable_re}(?!\cH))+)//) {
 	    my $s = $1;
 	    if ((my $w = vwidth($s)) <= $room) {
 		$folded .= $s;
@@ -570,7 +569,7 @@ Text::ANSI::Fold - Text folding library supporting ANSI terminal sequence and As
 
 =head1 VERSION
 
-Version 2.24
+Version 2.25
 
 =head1 SYNOPSIS
 
@@ -595,7 +594,7 @@ Version 2.24
         linebreak => LINEBREAK_ALL,
         runin     => 4,
         runout    => 4,
-        );
+    );
 
 =head1 DESCRIPTION
 
@@ -615,6 +614,8 @@ Since the overhead of ANSI escape sequence handling is not significant
 when the data does not include them, this module can be used for
 normal text processing with small penalty.
 
+=head2 B<ansi_fold>
+
 Use exported B<ansi_fold> function to fold original text, with number
 of visual columns you want to cut off the text.
 
@@ -635,6 +636,8 @@ This function returns at least one character in any situation.  If you
 provide Asian wide string and just one column as width, it trims off
 the first wide character even if it does not fit to given width.
 
+=head2 B<configure>
+
 Default parameter can be set by B<configure> class method:
 
     Text::ANSI::Fold->configure(width => 80, padding => 1);
@@ -654,49 +657,74 @@ L<Text::ANSI::Fold::Util>.
 =head1 OBJECT INTERFACE
 
 You can create an object to hold parameters, which is effective during
-object life time.  For example,
+object life time.
 
-    my $f = Text::ANSI::Fold->new(
+=head2 B<new>
+
+Use C<new> method to make a fold object.
+
+    my $obj = Text::ANSI::Fold->new(
         width => 80,
         boundary => 'word',
-        );
+    );
 
-makes an object folding on word boundaries with 80 columns width.
-Then you can use this without parameters.
+This makes an object folding on word boundaries with 80 columns width.
 
-    $f->fold($text);
+=head2 B<fold>
+
+Then you can call C<fold> method without parameters because the object
+keeps necessary information.
+
+    $obj->fold($text);
+
+=head2 B<configure>
 
 Use B<configure> method to update parameters:
 
-    $f->configure(padding => 1);
+    $obj->configure(padding => 1);
 
 Additional parameter can be specified on each call, and they precede
 saved value.
 
-    $f->fold($text, width => 40);
+    $obj->fold($text, width => 40);
 
 =head1 STRING OBJECT INTERFACE
 
+You can use a fold object to hold string inside, and take out folded
+strings from it.  Use C<text> method to store data in the object, and
+C<retrieve> or C<chops> method to take out folded string from it.
+
+=head2 B<text>
+
 A fold object can hold string inside by B<text> method.
 
-    $f->text("text");
+    $obj->text("text");
 
-And folded string can be taken by B<retrieve> method.  It returns
+Method B<text> has an lvalue attribute, so it can be assigned to, as
+well as can be a subject of mutating operator such as C<s///>.
+
+    $obj->text = "text";
+
+=head2 B<retrieve>
+
+Folded string can be taken out by C<retrieve> method.  It returns
 empty string if nothing remained.
 
-    while ((my $folded = $f->retrieve) ne '') {
+    while ((my $folded = $obj->retrieve) ne '') {
         print $folded;
         print "\n" if $folded !~ /\n\z/;
     }
 
-Method B<chops> returns chopped string list.  Because the B<text>
+=head2 B<chops>
+
+Method C<chops> returns chopped string list.  Because the C<text>
 method returns the object itself when called with a parameter, you can
-use B<text> and B<chops> in series:
+use C<text> and C<chops> in series:
 
-    print join "\n", $f->text($string)->chops;
+    print join "\n", $obj->text($string)->chops;
 
-Actually, text can be set by B<new> or B<configure> method through
-B<text> parameter.  Next program just works.
+Actually, text can be set by c<new> or C<configure> method through
+C<text> parameter.  Next program just works.
 
     use Text::ANSI::Fold;
     while (<>) {
@@ -704,7 +732,9 @@ B<text> parameter.  Next program just works.
             Text::ANSI::Fold->new(width => 40, text => $_)->chops;
     }
 
-When using B<chops> method, B<width> parameter can take array
+=head2 B<chops> with multiple width
+
+When using C<chops> method, C<width> parameter can take array
 reference, and chops text into given width list.
 
     my $fold = Text::ANSI::Fold->new;
@@ -712,10 +742,13 @@ reference, and chops text into given width list.
     # return ("1", "22", "333") and keep "4444"
 
 If the width value is 0, it returns empty string.  Negative width
-value takes all the rest of stored string.
+value takes all the rest of stored string.  In the following code, the
+fourth width (3) is ignored because the -2 immediately preceding it
+consumes all remaining strings.
 
-Method B<text> has an lvalue attribute, so it can be assigned to, as
-well as can be a subject of mutating operator such as C<s///>.
+    my $fold = Text::ANSI::Fold->new;
+    my @list = $fold->text("1223334444")->chops(width => [ 1, 0, -2, 3 ]);
+    # return ("1", "", "223334444")
 
 =head1 OPTIONS
 
@@ -726,9 +759,9 @@ function as well as B<new> and B<configure> method.
 
     Text::ANSI::Fold->configure(boundary => 'word');
 
-    my $f = Text::ANSI::Fold->new(boundary => 'word');
+    my $obj = Text::ANSI::Fold->new(boundary => 'word');
 
-    $f->configure(boundary => 'word');
+    $obj->configure(boundary => 'word');
 
 =over 7
 
@@ -906,13 +939,56 @@ text with Japanese prohibited character handling.
         linebreak => LINEBREAK_ALL,
         runin     => 4,
         runout    => 4,
-        );
+    );
     
     $, = "\n";
     while (<>) {
         print $fold->text($_)->chops;
     }
 
+=head1 CONTROL CHARACTERS
+
+Text::ANSI::Fold handles the following line break related codes
+specially:
+
+    Newline (\n)
+    Form feed (\f)
+    Carriage return (\r)
+    Null (\0)
+    Line separator (U+2028)
+    Paragraph separator (U+2029)
+
+These characters are handled as follows:
+
+=over 4
+
+=item NEWLINE (C<\n>), CRNL (C<\r\n>)
+
+=item NULL (C<\0>)
+
+=item LINE SEPARATOR (C<U+2028>)
+
+=item PARAGRAPH SEPARATOR (C<U+2029>)
+
+If any of these characters are found, the folding process terminates
+immediately and the portion up to that character is returned as folded
+text.  These character itself is included at the end of the folded
+text.
+
+=item CARRIAGE RETURN (C<\r>)
+
+When a carriage return is found, it is added to the folded text and
+the fold width is reset.
+
+=item FORM FEED (C<\f>)
+
+If a form feed character is found in the middle of a string,
+processing stops and the string up to the point immediately before is
+returned.  If it is found at the beginning of a string, it is added to
+the folded text and processing continues.
+
+=back
+
 =head1 SEE ALSO
 
 =over 7
